#include <iostream>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>

#include "playertask.hh"

playertask::playertask(int s) : sock(s), lineterm(0)
{
	cout << "[" << sock << "]\n";
	echo("*** You have been noticed.");
}

playertask::~playertask()
{
	cout << "[~" << sock << "]\n";
	close(sock);
}

void playertask::echo(const string &s)
{
	fcntl(sock, F_SETFL, fcntl(sock, F_GETFL) &~ O_NONBLOCK);
	
	const char *q = s.c_str();
	unsigned c = s.size();
	
	while(c)
	{
		int n = write(sock, q, c);
		if(n < 0)break;
		q += n;
		c -= n;			
	}
	
	write(sock, "\r\n", 2);
	
	fcntl(sock, F_SETFL, fcntl(sock, F_GETFL) | O_NONBLOCK);
}

void playertask::handleinput()
{
	cout << "[" << sock << "]: " << input << endl;
}

bool playertask::tick()
{
	unsigned char ch;
	int n;
	while((n = read(sock, &ch, 1)) == 1)
	{
		switch(ch)
		{
			case '':
			case '':
				if(input.size() > 0)input = input.substr(0, input.size()-1);
				break;
			case '\r':
				if(lineterm=='\n')break;
				handleinput();
				input = "";
				lineterm = '\r';
				break;
			case '\n':
				if(lineterm=='\r')break;
				handleinput();
				input = "";
				lineterm = '\n';
				break;
			default:
				input += ch;					
		}
	}
	if(!n)return false; /* Socket died */
	if(n < 0 && errno != EAGAIN && errno != EINTR)
	{
		extern const char *sys_errlist[];
		cerr << "[!" << sock << "]: " << sys_errlist[errno] << endl;
		return false;
	}
	return true;
}
